@@ -35,13 +35,14 @@
 
 #include "nautilus-window-private.h"
 #include "nautilus-window.h"
+#include <eel/eel-accessibility.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
+#include <eel/eel-input-event-box.h>
 #include <eel/eel-vfs-extensions.h>
 #include <gtk/gtkdnd.h>
-#include <gtk/gtkeventbox.h>
 #include <gtk/gtksignal.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-stock-icons.h>
@@ -125,6 +126,8 @@ drag_data_received_callback (GtkWidget *widget,
 	NautilusApplication *application;
 	int name_count;
 	NautilusWindow *new_window;
+	NautilusWindow *window;
+	GdkScreen      *screen;
 	gboolean new_windows_for_extras;
 	char *prompt;
 
@@ -140,6 +143,7 @@ drag_data_received_callback (GtkWidget *widget,
 		return;
 	}
 
+	window = nautilus_location_bar_get_window (widget);
 	new_windows_for_extras = FALSE;
 	/* Ask user if they really want to open multiple windows
 	 * for multiple dropped URIs. This is likely to have been
@@ -153,7 +157,7 @@ drag_data_received_callback (GtkWidget *widget,
 		/* eel_run_simple_dialog should really take in pairs
 		 * like gtk_dialog_new_with_buttons() does. */
 		new_windows_for_extras = eel_run_simple_dialog 
-			(GTK_WIDGET (nautilus_location_bar_get_window (widget)),
+			(GTK_WIDGET (window),
 			 TRUE,
 			 prompt,
 			 _("View in Multiple Windows?"),
@@ -173,9 +177,11 @@ drag_data_received_callback (GtkWidget *widget,
 	nautilus_navigation_bar_location_changed (NAUTILUS_NAVIGATION_BAR (widget));
 
 	if (new_windows_for_extras) {
-		application = nautilus_location_bar_get_window (widget)->application;
+		application = window->application;
+		screen = gtk_window_get_screen (GTK_WINDOW (window));
+
 		for (node = names->next; node != NULL; node = node->next) {
-			new_window = nautilus_application_create_window (application);
+			new_window = nautilus_application_create_window (application, screen);
 			nautilus_window_go_to (new_window, node->data);
 		}
 	}
@@ -392,6 +398,13 @@ try_to_expand_path (gpointer callback_data)
 		return FALSE;
 	}
 
+	/* We already completed if we have a trailing '/' */
+	if (current_path[strlen (current_path) - 1] == GNOME_VFS_URI_PATH_CHR) {
+		g_free (user_location);
+		g_free (current_path);
+		return FALSE;
+	}
+
 	user_location_length = g_utf8_strlen (user_location, -1);
 
 	g_free (user_location);
@@ -414,12 +427,14 @@ try_to_expand_path (gpointer callback_data)
 
 	dir_name = gnome_vfs_uri_extract_dirname (uri);
 
+	gnome_vfs_uri_unref (uri);
+	uri = NULL;
+
 	/* get file info for the directory, if it hasn't changed since last time */
 	get_file_info_list (bar, dir_name);
 	if (bar->details->file_info_list == NULL) {
 		g_free (dir_name);
 		g_free (base_name);
-		gnome_vfs_uri_unref (uri);
 		g_free (current_path);
 		return FALSE;
 	}
@@ -444,7 +459,7 @@ try_to_expand_path (gpointer callback_data)
 		}
 	}
 
-	if (have_broken_filenames()) {
+	if (have_broken_filenames ()) {
 		if (expand_text) {
 			expand_text_utf8 = g_locale_to_utf8 (expand_text, -1, NULL, NULL, NULL);
 			g_free (expand_text);
@@ -570,16 +585,13 @@ editable_event_after_callback (GtkEntry *entry,
 	bar = NAUTILUS_LOCATION_BAR (user_data);
 
 	/* After typing the right arrow key we move the selection to
-	 * the end.
+	 * the end, if we have a valid selection - since this is most
+	 * likely an auto-completion. We ignore shift / control since
+	 * they can validly be used to extend the selection.
 	 */
-	/* FIXME: Why do this? This breaks key combinations like
-	 * Ctrl-Right to move a word at a time and Shift-Right to
-	 * extend the selection by a single character, and also means
-	 * you can't move through the text a character at a
-	 * time. Seems like a bad idea.
-	 */
-	if ((keyevent->keyval == GDK_Right || keyevent->keyval == GDK_End)
-	    && gtk_editable_get_selection_bounds (editable, NULL, NULL)) {
+	if ((keyevent->keyval == GDK_Right || keyevent->keyval == GDK_End) &&
+	    !(keyevent->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) && 
+	    gtk_editable_get_selection_bounds (editable, NULL, NULL)) {
 		set_position_and_selection_to_end (editable);
 	}
 
@@ -591,7 +603,7 @@ editable_event_after_callback (GtkEntry *entry,
 	if (position_and_selection_are_at_end (editable)) {
 		if (entry_would_have_inserted_characters (keyevent)) {
 			if (bar->details->idle_id == 0) {
-				bar->details->idle_id = gtk_idle_add (try_to_expand_path, bar);
+				bar->details->idle_id = g_idle_add (try_to_expand_path, bar);
 			}
 		}
 	} else {
@@ -599,7 +611,7 @@ editable_event_after_callback (GtkEntry *entry,
 		 * to change the position or selection.
 		 */
 		if (bar->details->idle_id != 0) {
-			gtk_idle_remove (bar->details->idle_id);
+			g_source_remove (bar->details->idle_id);
 			bar->details->idle_id = 0;
 		}
 	}
@@ -642,7 +654,7 @@ destroy (GtkObject *object)
 	
 	/* cancel the pending idle call, if any */
 	if (bar->details->idle_id != 0) {
-		gtk_idle_remove (bar->details->idle_id);
+		g_source_remove (bar->details->idle_id);
 		bar->details->idle_id = 0;
 	}
 	
@@ -692,7 +704,7 @@ nautilus_location_bar_init (NautilusLocationBar *bar)
 
 	hbox = gtk_hbox_new (0, FALSE);
 
-	event_box = gtk_event_box_new ();
+	event_box = eel_input_event_box_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (event_box),
 					GNOME_PAD_SMALL);
 	label = gtk_label_new (LOCATION_LABEL);
@@ -717,6 +729,8 @@ nautilus_location_bar_init (NautilusLocationBar *bar)
 
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 
+	eel_accessibility_set_up_label_widget_relation (label, entry);
+
 	gtk_container_add (GTK_CONTAINER (bar), hbox);
 
 	/* Drag source */
